	.text
	.align 4
	.global	to0
to0:
	.byte 0
	.global	to1
to1:
	.byte 0
	.global	to2
to2:
	.byte 0
	.global	to3
to3:
	.byte 0
	.global	to4
to4:
	.byte 0
	.byte 0

	.data
	.align 4
	.global	do0
do0:
	.byte 0
	.global	do1
do1:
	.byte 0
	.global	do2
do2:
	.byte 0
	.global	do3
do3:
	.byte 0
	.global	do4
do4:
	.byte 0
