#objdump: -rs
#name: pushpop-word-sreg-dcr1-special

.*:     file format .*-cris

Contents of section \.text:
 0000 fee17e7e fee17e7e fee17e7e f8e17e7e  .*
 0010 fbe17e7e fce17e7e fde17e7e ffe17e7e  .*
 0020 00e17e7e 01e17e7e 02e17e7e 03e17e7e  .*
 0030 04e17e7e 05e17e7e 08e17e7e 08e13e7e  .*
 0040 05e13e7e 04e13e7e 03e13e7e 02e13e7e  .*
 0050 01e13e7e 00e13e7e ffe13e7e fee13e7e  .*
 0060 fde13e7e fce13e7e fbe13e7e f8e13e7e  .*
 0070 fee17e7e 3e7e3e7e fee17e7e           .*
Contents of section \.data:
