# objdump: -dtr

.*:     file format elf64-mmix

SYMBOL TABLE:
0+ l    d  \.text	0+ 
0+ l    d  \.data	0+ 
0+ l    d  \.bss	0+ 
0+ g     F \.text	0+ Main


Disassembly of section \.text:

0+ <Main>:
   0:	220116e9 	addu \$1,\$22,\$233
