[^:]*: Assembler messages:
[^:]*:4: Error: garbage following instruction -- `fstd d0,\[r0\],#8'
[^:]*:5: Error: garbage following instruction -- `fstd d0,\[r0,#-8\]!'
[^:]*:6: Error: garbage following instruction -- `fsts s0,\[r0\],#8'
[^:]*:7: Error: garbage following instruction -- `fsts s0,\[r0,#-8\]!'
[^:]*:8: Error: garbage following instruction -- `fldd d0,\[r0\],#8'
[^:]*:9: Error: garbage following instruction -- `fldd d0,\[r0,#-8\]!'
[^:]*:10: Error: garbage following instruction -- `flds s0,\[r0\],#8'
[^:]*:11: Error: garbage following instruction -- `flds s0,\[r0,#-8\]!'
