#objdump: -Dr
#source: immediate-003.s

.*:     file format elf32-d10v

Disassembly of section .text:

00000000 <foo>:
   0:	e0 00 00 00 	ldi.l	r0, 0x0
			2: R_D10V_16	sym
Disassembly of section .data:
