101 DIALOG DISCARDABLE 0, 0, 186, 95
FONT 8, "Tahoma"
BEGIN
    DEFPUSHBUTTON "OK", 1, 129, 7, 50, 14
END

102 DIALOGEX DISCARDABLE 0, 0, 186, 95
FONT 8, "Tahoma"
BEGIN
    DEFPUSHBUTTON "OK", 1, 129, 7, 50, 14
END

103 DIALOGEX DISCARDABLE 0, 0, 186, 95
FONT 8, "Tahoma", 0
BEGIN
    DEFPUSHBUTTON "OK", 1, 129, 7, 50, 14
END

104 DIALOGEX DISCARDABLE 0, 0, 186, 95
FONT 8, "Tahoma", 0, 0
BEGIN
    DEFPUSHBUTTON "OK", 1, 129, 7, 50, 14
END

105 DIALOGEX DISCARDABLE 0, 0, 186, 95
FONT 8, "Tahoma", 0, 0, 1
BEGIN
    DEFPUSHBUTTON "OK", 1, 129, 7, 50, 14
END
