! PT, PTA, PTB expansion for external symbols.
start:
	pt	externalsym1 + 40,tr5
	pta	externalsym2 + 44,tr4
	ptb	externalsym3 + 48,tr3
	pt/u	externalsym4 + 52,tr5
	pta/u	externalsym5 + 56,tr4
	ptb/u	externalsym6 + 60,tr3
