ARCH=arm
SCRIPT_NAME=pe
OUTPUT_FORMAT="pei-arm-little"
LITTLE_OUTPUT_FORMAT="pei-arm-little"
BIG_OUTPUT_FORMAT="pei-arm-big"
TEMPLATE_NAME=pe
ENTRY="_mainCRTStartup"
SUBSYSTEM=PE_DEF_SUBSYSTEM
INITIAL_SYMBOL_CHAR=\"_\"
