	.text
	.am33
	mov 0x1ffeff,r2
	movu 0x1ffeff,r2
	add 0x1ffeff,r2
	addc 0x1ffeff,r2
	sub 0x1ffeff,r2
	subc 0x1ffeff,r2
	cmp 0x1ffeff,r2
	mov 0x1ffeff,xr2
	and 0x1ffeff,r2
	or 0x1ffeff,r2
	xor 0x1ffeff,r2
	asr 0x1ffeff,r2
	lsr 0x1ffeff,r2
	asl 0x1ffeff,r2
	mul 0x1ffeff,r2
	mulu 0x1ffeff,r2
	btst 0x1ffeff,r2
	mov (0x1ffeff,r1),r2
	mov r2,(0x1ffeff,r1)
	movbu (0x1ffeff,r1),r2
	movbu r2,(0x1ffeff,r1)
	movhu (0x1ffeff,r1),r2
	movhu r2,(0x1ffeff,r1)
	mov (0x1ffeff,sp),r2
	mov r2,(0x1ffeff,sp)
	movbu (0x1ffeff,sp),r2
	movbu r2,(0x1ffeff,sp)
	movhu (0x1ffeff,sp),r2
	movhu r2,(0x1ffeff,sp)
	mac 0x1ffeff,r2
	macu 0x1ffeff,r2
	macb 0x1ffeff,r2
	macbu 0x1ffeff,r2
	mach 0x1ffeff,r2
	machu 0x1ffeff,r2
	mov (0x1ffeff),r2
	mov r2,(0x1ffeff)
	movbu (0x1ffeff),r2
	movbu r2,(0x1ffeff)
	movhu (0x1ffeff),r2
	movhu r2,(0x1ffeff)
	mov (r1+,0x1ffeff),r2
	mov r1,(r2+,0x1ffeff)
	movhu (r1+,0x1ffeff),r2
	movhu r1,(r2+,0x1ffeff)
