	.text
	.globl F
F:
	mvfs	f0, f0
	mvfsp	f0, f0
	mvfsm	f0, f0
	mvfsz	f0, f0
	mvfd	f0, f0
	mvfdp	f0, f0
	mvfdm	f0, f0
	mvfdz	f0, f0
	mvfe	f0, f0
	mvfep	f0, f0
	mvfem	f0, f0
	mvfez	f0, f0

	mnfs	f0, f0
	mnfsp	f0, f0
	mnfsm	f0, f0
	mnfsz	f0, f0
	mnfd	f0, f0
	mnfdp	f0, f0
	mnfdm	f0, f0
	mnfdz	f0, f0
	mnfe	f0, f0
	mnfep	f0, f0
	mnfem	f0, f0
	mnfez	f0, f0

	abss	f0, f0
	abssp	f0, f0
	abssm	f0, f0
	abssz	f0, f0
	absd	f0, f0
	absdp	f0, f0
	absdm	f0, f0
	absdz	f0, f0
	abse	f0, f0
	absep	f0, f0
	absem	f0, f0
	absez	f0, f0

	rnds	f0, f0
	rndsp	f0, f0
	rndsm	f0, f0
	rndsz	f0, f0
	rndd	f0, f0
	rnddp	f0, f0
	rnddm	f0, f0
	rnddz	f0, f0
	rnde	f0, f0
	rndep	f0, f0
	rndem	f0, f0
	rndez	f0, f0

	sqts	f0, f0
	sqtsp	f0, f0
	sqtsm	f0, f0
	sqtsz	f0, f0
	sqtd	f0, f0
	sqtdp	f0, f0
	sqtdm	f0, f0
	sqtdz	f0, f0
	sqte	f0, f0
	sqtep	f0, f0
	sqtem	f0, f0
	sqtez	f0, f0

	logs	f0, f0
	logsp	f0, f0
	logsm	f0, f0
	logsz	f0, f0
	logd	f0, f0
	logdp	f0, f0
	logdm	f0, f0
	logdz	f0, f0
	loge	f0, f0
	logep	f0, f0
	logem	f0, f0
	logez	f0, f0

	lgns	f0, f0
	lgnsp	f0, f0
	lgnsm	f0, f0
	lgnsz	f0, f0
	lgnd	f0, f0
	lgndp	f0, f0
	lgndm	f0, f0
	lgndz	f0, f0
	lgne	f0, f0
	lgnep	f0, f0
	lgnem	f0, f0
	lgnez	f0, f0

	exps	f0, f0
	expsp	f0, f0
	expsm	f0, f0
	expsz	f0, f0
	expd	f0, f0
	expdp	f0, f0
	expdm	f0, f0
	expdz	f0, f0
	expe	f0, f0
	expep	f0, f0
	expem	f0, f0
	expdz	f0, f0

	sins	f0, f0
	sinsp	f0, f0
	sinsm	f0, f0
	sinsz	f0, f0
	sind	f0, f0
	sindp	f0, f0
	sindm	f0, f0
	sindz	f0, f0
	sine	f0, f0
	sinep	f0, f0
	sinem	f0, f0
	sinez	f0, f0

	coss	f0, f0
	cossp	f0, f0
	cossm	f0, f0
	cossz	f0, f0
	cosd	f0, f0
	cosdp	f0, f0
	cosdm	f0, f0
	cosdz	f0, f0
	cose	f0, f0
	cosep	f0, f0
	cosem	f0, f0
	cosez	f0, f0

	tans	f0, f0
	tansp	f0, f0
	tansm	f0, f0
	tansz	f0, f0
	tand	f0, f0
	tandp	f0, f0
	tandm	f0, f0
	tandz	f0, f0
	tane	f0, f0
	tanep	f0, f0
	tanem	f0, f0
	tanez	f0, f0

	asns	f0, f0
	asnsp	f0, f0
	asnsm	f0, f0
	asnsz	f0, f0
	asnd	f0, f0
	asndp	f0, f0
	asndm	f0, f0
	asndz	f0, f0
	asne	f0, f0
	asnep	f0, f0
	asnem	f0, f0
	asnez	f0, f0

	acss	f0, f0
	acssp	f0, f0
	acssm	f0, f0
	acssz	f0, f0
	acsd	f0, f0
	acsdp	f0, f0
	acsdm	f0, f0
	acsdz	f0, f0
	acse	f0, f0
	acsep	f0, f0
	acsem	f0, f0
	acsez	f0, f0

	atns	f0, f0
	atnsp	f0, f0
	atnsm	f0, f0
	atnsz	f0, f0
	atnd	f0, f0
	atndp	f0, f0
	atndm	f0, f0
	atndz	f0, f0
	atne	f0, f0
	atnep	f0, f0
	atnem	f0, f0
	atnez	f0, f0

	urds	f0, f0
	urdsp	f0, f0
	urdsm	f0, f0
	urdsz	f0, f0
	urdd	f0, f0
	urddp	f0, f0
	urddm	f0, f0
	urddz	f0, f0
	urde	f0, f0
	urdep	f0, f0
	urdem	f0, f0
	urdez	f0, f0

	nrms	f0, f0
	nrmsp	f0, f0
	nrmsm	f0, f0
	nrmsz	f0, f0
	nrmd	f0, f0
	nrmdp	f0, f0
	nrmdm	f0, f0
	nrmdz	f0, f0
	nrme	f0, f0
	nrmep	f0, f0
	nrmem	f0, f0
	nrmez	f0, f0
