# objdump: -dr
# as: -linkrelax
# source: resume-op.s
.*:     file format elf64-mmix

Disassembly of section \.text:

0000000000000000 <Main>:
   0:	f9000001 	resume 1
   4:	f9000000 	resume 0
   8:	f9000001 	resume 1
   c:	f9000000 	resume 0
