.*: Assembler messages:
.*:42: Warning: Macro instruction expanded into multiple instructions in a branch delay slot
.*:56: Warning: Macro instruction expanded into multiple instructions in a branch delay slot
