% Get rid of labels that look compiler-generated, matching: "L.*:[0-9]+".
% Only if they're local, of course.
	.global	L:21
	.global	LC:23
L:9 SWYM 0
L:12 SWYM 1
L:21 SWYM 2
LC:32 SWYM 3
LC:23 SWYM 4
L9 SWYM 10
L12 SWYM 11
L21 SWYM 12
LC32 SWYM 13
LC23 SWYM 14
