[^:]*: Assembler messages:
[^:]*:4: Error: invalid pseudo operation -- `str r0,=0x00ff0000'
[^:]*:5: Error: bad expression -- `ldr r0,{r1}'
[^:]*:6: Error: address offset too large -- `ldr r0,\[r1,#4096\]'
[^:]*:7: Error: address offset too large -- `ldr r0,\[r1,#-4096\]'
[^:]*:8: Error: invalid constant -- `mov r0,#0x1ff'
[^:]*:9: Error: bad instruction `cmpl r0,r0'
[^:]*:10: Error: selected processor does not support `strh r0,\[r1\]'
[^:]*:11: Warning: writeback of base register is UNPREDICTABLE
[^:]*:12: Warning: writeback of base register when in register list is UNPREDICTABLE
[^:]*:13: Warning: writeback of base register is UNPREDICTABLE
[^:]*:15: Warning: if writeback register is in list, it must be the lowest reg in the list
