# objdump: -t

.*:     file format elf64-mmix

SYMBOL TABLE:
0+ l    d  \.text	0+ 
0+ l    d  \.data	0+ 
0+ l    d  \.bss	0+ 
0+14 l       \.text	0+ L9
0+18 l       \.text	0+ L12
0+1c l       \.text	0+ L21
0+20 l       \.text	0+ LC32
0+24 l       \.text	0+ LC23
0+8 g       \.text	0+ L:21
0+10 g       \.text	0+ LC:23
