#readelf: -wl
#source: unimplemented.s
#as: --em=criself --gdwarf2

# Prefixes of various sizes.
#...
 Line Number Statements:
  Extended opcode 2: set Address to 0x0
  Special opcode 13: advance Address by 0 to 0x0 and Line by 8 to 9
  Special opcode 20: advance Address by 2 to 0x2 and Line by 1 to 10
  Special opcode 34: advance Address by 4 to 0x6 and Line by 1 to 11
  Special opcode 34: advance Address by 4 to 0xa and Line by 1 to 12
  Special opcode 62: advance Address by 8 to 0x12 and Line by 1 to 13
  Special opcode 62: advance Address by 8 to 0x1a and Line by 1 to 14
  Special opcode 20: advance Address by 2 to 0x1c and Line by 1 to 15
  Special opcode 34: advance Address by 4 to 0x20 and Line by 1 to 16
  Special opcode 34: advance Address by 4 to 0x24 and Line by 1 to 17
  Special opcode 62: advance Address by 8 to 0x2c and Line by 1 to 18
  Special opcode 62: advance Address by 8 to 0x34 and Line by 1 to 19
  Special opcode 20: advance Address by 2 to 0x36 and Line by 1 to 20
  Special opcode 20: advance Address by 2 to 0x38 and Line by 1 to 21
  Special opcode 20: advance Address by 2 to 0x3a and Line by 1 to 22
  Special opcode 20: advance Address by 2 to 0x3c and Line by 1 to 23
  Special opcode 20: advance Address by 2 to 0x3e and Line by 1 to 24
  Special opcode 20: advance Address by 2 to 0x40 and Line by 1 to 25
  Special opcode 20: advance Address by 2 to 0x42 and Line by 1 to 26
  Special opcode 20: advance Address by 2 to 0x44 and Line by 1 to 27
  Special opcode 20: advance Address by 2 to 0x46 and Line by 1 to 28
  Advance PC by 2 to 48
  Extended opcode 1: End of Sequence
