# objdump: -dr
# as: -linkrelax
# source: save-op.s
.*:     file format elf64-mmix

Disassembly of section \.text:

0000000000000000 <Main>:
   0:	fa2d0000 	save \$45,0
   4:	fa1f0000 	save \$31,0
   8:	fa000000 	save \$0,0
