#objdump: -str

.*:     file format elf64-mmix

SYMBOL TABLE:
0+ l    d  \.text	0+ 
0+ l    d  \.data	0+ 
0+ l    d  \.bss	0+ 
0+ g     F \.text	0+ Main

Contents of section \.text:
 0000 61623b00 00000000 00000000 00000064  .*
 0010 00000000 00000065                    .*
Contents of section \.data:
