! Section with SHmedia in unique section.  Note the absence of a symbol to
! key an ISA type.

	.section .text.2f,"ax"
	.align 2

	.mode SHmedia
	movi	0x2f,r20
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	movi	0x2f,r21
