. ${srcdir}/emulparams/elf32bmipn32-defs.sh
OUTPUT_FORMAT="elf32-nbigmips"
BIG_OUTPUT_FORMAT="elf32-nbigmips"
LITTLE_OUTPUT_FORMAT="elf32-nlittlemips"
GENERATE_SHLIB_SCRIPT=yes
SHLIB_TEXT_START_ADDR=0x5ffe0000
