/* RISC/os 4.52C, and presumably other versions.  */

#include <bsd43/machine/machparam.h>
#include <bsd43/machine/vmparam.h>

#define NBPG BSD43_NBPG
#define UPAGES BSD43_UPAGES
#define HOST_TEXT_START_ADDR BSD43_USRTEXT
#define HOST_DATA_START_ADDR BSD43_USRDATA
#define HOST_STACK_END_ADDR BSD43_USRSTACK
