 LOCAL extreg
 LOCAL reghere
 LOCAL consthere
 LOCAL 100
 LOCAL $200

reghere IS $144
consthere IS 45

 # Restrictions on the implementation means we have to have the LOCAL in
 # code or data.
 SWYM 0,0,0
