.RADIX Q
	! test floats now
	.float -H'C
	.float -9.0
	.float 0.9
	.float +0.9
	.float +.9
	.float -00.0009
	.float -2.900
	.float 1.1e+10
	.float +1.1e+10
	.float +1.1e-10
	.float -1.1E+10
	.float +1.1E+1000
	.float +1.1E+0001 10
	.float 11.11E-10 10
	.float H'C.3
	.float H'C.H'C
.END
