.*: Assembler messages:
.*:6: Error: Invalid opcode/operands
.*:7: Error: Unrecognized opcode: .*
.*:7: Error: Invalid opcode/operands
.*:8: Error: Unrecognized opcode: .*
.*:8: Error: Invalid opcode/operands
.*:9: Error: Unrecognized opcode: .*
.*:9: Error: Invalid opcode/operands
.*:10: Error: Unrecognized opcode: .*
.*:10: Error: Invalid opcode/operands
.*:11: Error: Unrecognized opcode: .*
.*:11: Error: Invalid opcode/operands
.*:12: Error: Unrecognized opcode: .*
.*:12: Error: Invalid opcode/operands
.*:13: Error: Unrecognized opcode: .*
.*:13: Error: Invalid opcode/operands
.*:14: Error: Unrecognized opcode: .*
.*:14: Error: Invalid opcode/operands
.*:15: Error: Unrecognized opcode: .*
.*:15: Error: Invalid opcode/operands
.*:16: Error: Unrecognized opcode: .*
.*:16: Error: Invalid opcode/operands
.*:17: Error: Unrecognized opcode: .*
.*:17: Error: Invalid opcode/operands
.*:18: Error: Unrecognized opcode: .*
