% The .text contents is supposed to be linked --oformat binary with
% b-twoinsn.s.  It will provide an incorrect LOP_END; its YZ field is not
% the number of tetras to the preceding LOP_STAB.
 .text
 .byte 0x98,0x0b,0x00,0x00,0x20,0x3a,0x30,0x4d,0x20,0x61,0x20,0x69
 .byte 0x01,0x6e,0x04,0x81,0x01,0x61,0x04,0x82,0x98,0x0c,0x00,0x03
