	.SPACE $PRIVATE$
	.SUBSPA $DATA$,QUAD=1,ALIGN=8,ACCESS=31
	.SPACE $TEXT$
	.SUBSPA $CODE$,QUAD=0,ALIGN=8,ACCESS=44,CODE_ONLY

	.blockz 4

main:
	.SPACE $PRIVATE$
	.SUBSPA $DATA$

	.SPACE $TEXT$
	.SUBSPA $CODE$

	.PROC
	.CALLINFO FRAME=0
	.ENTRY
	nop
	.EXIT
	.PROCEND
