#readelf: -wl
#source: continue.s
#as: --em=criself --gdwarf2

# Continued line.
#...
 Line Number Statements:
  Extended opcode 2: set Address to 0x0
  Special opcode 10: advance Address by 0 to 0x0 and Line by 5 to 6
  Advance PC by 4 to 4
  Extended opcode 1: End of Sequence
